"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const task_scheduler_1 = require("@microsoft/task-scheduler");
const NpmScriptTask_1 = require("./NpmScriptTask");
const getPipelinePackages_1 = require("./getPipelinePackages");
const parsePipelineConfig_1 = require("./parsePipelineConfig");
/** Returns a map that maps task name to the priorities config for that task */
function getPriorityMap(priorities) {
    const result = new Map();
    priorities.forEach((entry) => {
        const taskPriority = result.get(entry.task) || {};
        taskPriority[entry.package] = entry.priority;
        result.set(entry.task, taskPriority);
    });
    return result;
}
async function runTasks(options) {
    const { graph, workspace, context, config } = options;
    const priorityMap = getPriorityMap(config.priorities);
    let pipeline = task_scheduler_1.createPipeline(graph, {
        // dummy logger for task-scheduler because lage already has logger for its tasks
        logger: {
            error: (_msg) => { },
            log: (_msg) => { },
        },
        exit: (code) => { },
        targetsOnly: config.only,
        concurrency: config.concurrency,
    });
    const pipelineConfig = parsePipelineConfig_1.parsePipelineConfig(config.pipeline);
    // Add in the task deps based on package graph
    for (const [taskName, taskDeps] of Object.entries(pipelineConfig.taskDeps)) {
        const { deps, topoDeps } = taskDeps;
        pipeline = pipeline.addTask({
            name: taskName,
            deps,
            topoDeps,
            priorities: priorityMap.get(taskName),
            run: async (_location, _stdout, _stderr, pkg) => {
                const info = workspace.allPackages[pkg];
                const scripts = info.scripts;
                if (scripts && scripts[taskName]) {
                    const npmTask = new NpmScriptTask_1.NpmScriptTask(taskName, workspace.root, info, config, context);
                    context.tasks.set(task_scheduler_1.getTaskId(info.name, taskName), npmTask);
                    return await npmTask.run();
                }
                return true;
            },
        });
    }
    // adding specific package task dependencies
    for (const packageTaskDep of pipelineConfig.packageTaskDeps) {
        const from = task_scheduler_1.getPackageTaskFromId(packageTaskDep[0]);
        const to = task_scheduler_1.getPackageTaskFromId(packageTaskDep[1]);
        pipeline.addDep({ package: from[0], task: from[1] }, { package: to[0], task: to[1] });
    }
    await pipeline.go({
        packages: getPipelinePackages_1.getPipelinePackages(workspace, config),
        tasks: config.command,
    });
}
exports.runTasks = runTasks;
